/* ------------------------------------------------------------------------------------
 * Proprits du recouvrement par execXX()
 * Jean-Paul Rigault 
 *
 * Ce programme met en vidence les points suivants :
 *
 *	- aprs un fork() les buffers d'E/S de stdio sont hrits, mais ils
 *	  sont crass aprs exec()
 *	- exec() transforme le programme en cours d'excution en un autre programme 
 *	  (recouvrement) mais ne change pas de processus
 *
 * ------------------------------------------------------------------------------------
 */
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <dirent.h>

int main()
{
     /* Premier cas: aprs un fork(), les buffers d'E/S de stdio sont hrits
      * ---------------------------------------------------------------------
      */

    /* Le pre imprime un message, sans flush :
       Le message reste donc dans le buffer, sans tre crit sur stdout
     */
    printf("PID du pre : %d ", getpid()); /* noter absence de \n */

    switch (fork())
    {
    case -1: /* Erreur */
	perror("fork");
	abort();

    case 0: /* Processus fils */

	/* Affichage d'un message avec flush :
         *  on doit hriter (d'une copie) du buffer du pre et donc imprimer
         *  une premire fois "PID du pre ..." avant
        */
	printf("PID du fils = %d\n", getpid()); /* Ici on met \n */

	/* Fin du fils */
	exit(0);

    default: /* Suite du processus pre */

	/* Cette impression a pour effet secondaire de flusher le buffer du pre :
         * on doit donc imprimer une seconde fois "PID du pre ... " avant
        */
	printf("Continuation du pre\n");
	break;	   
    }


    /* Deuxime cas: aprs un exec(), les buffers d'E/S de stdio sont crass
     * ----------------------------------------------------------------------
     */ 

    /* On se place dans la mme situation: le pre imprime un message, sans flush */
    printf("PID du pre  nouveau :  = %d ", getpid()); 
				/* noter absence de \n : ce message va tre *perdu* !*/

    /* Le pre se transforme */
    execl("./exec_prop-aux.exe", "exec_prop-aux", NULL);
    perror("exec");	/* On ne devrait pas passer par ici */
    exit(0);		/* Fin du pre en cas d'erreur dans execl() */
}


