/* ------------------------------------------------------------------------------------
 * Proprits des processus fils
 * Jean-Paul Rigault 
 *
 * Ce programme met en vidence les points suivants :
 *
 *	- un processus fils a les mmes uid/gid rels/effectifs que son pre
 *	- un processus fils hrite des fichiers ouverts par son pre avec partage 
 *	  du pointeur d'E/S : bien que le pre et ls fils s'excutent en parallle,
 *	  leurs critures lmentaires (chaque appel  write) n'entrent pas en 
 *	  conflit, mais se mettent sagement  la suite les unes des autres
 *	  (en revanche on ne maitrise pas l'ordre)
 *	- un processus fils hrite aussi des rpertoires ouverts par son pre
 * 	- un processus orphelin est adopt par le processu de pid 1
 *
 * ------------------------------------------------------------------------------------
 */
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>  
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <fcntl.h>
#include <dirent.h>

char msg_parent[] = "Parent\n";
char msg_child[] = "Child\n";

int main()
{
  int fd;
  DIR *dirp;
  struct dirent *dentry;
  
  /* Affichage des uid/gid du pre */
  printf("Pre : uid = %d euid = %d gid = %d egid = %d\n",
	 getuid(), geteuid(), getgid(), getegid());
  
  /* Ouverture d'un rpertoire */
  if ((dirp = opendir(".")) == NULL) perror("opendir()");
  
  /* Ouverture d'un fichier et criture d'un message */
  fd = open("foo", O_WRONLY | O_CREAT | O_TRUNC, S_IRUSR | S_IWUSR);
  write(fd, msg_parent, strlen(msg_parent));
  
  switch (fork())
    {
    case -1: /* Erreur */
      perror("fork");
      abort();
      
    case 0: /* Processus fils */
      
      /* Affichage des uid/gid du fils */
      printf("Fils : uid = %d euid = %d gid = %d egid = %d\n",
	     getuid(), geteuid(), getgid(), getegid());
      
      /* Lecture du rpertoire ouvert par le pre */
      if ((dentry = readdir(dirp)) == NULL) perror("readdir()");
      else printf("Lu correctement rpertoire courant\n");
      
      /* criture d'un message dans le fichier hrit du pre :*/	
      write(fd, msg_child, strlen(msg_child));
      
      /* On attend que le pre se termine */
      sleep(2);
      
      /* criture d'un message dans le fichier hrit du pre  */	
      write(fd, msg_child, strlen(msg_child));
      
      /* Affichage du pid du pre (ce fils est devenu orphelin) */
      printf("PID du pre du fils orphelin : %d", getppid());
      
      /* Fin du fils */
      exit(0);
      
    default: /* Suite du processus pre */
      
      /* On attend afin que le fils puisse commencer  s'excuter */
      /* sleep(1); */
      
      /* criture d'un message dans le fichier commun */	
      write(fd, msg_parent, strlen(msg_parent));
      printf("Imprimer le contenu du fichier foo\n");
      
      /* Le pre se termine, rendant le fils orphelin */
      exit(0);
    }
  /*NOTREACHED*/
}


