/* ------------------------------------------------------------------------------------
 * Simulation de l'excution des commandes par le shell
 * Cette version utilise directement execXX()
 * Jean-Paul Rigault 
 * ------------------------------------------------------------------------------------
 */
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/wait.h>

int main()
{
    char *home;

    /* Excution de "who" */
    switch (fork())
    {
    case -1: /* Erreur */
	perror("fork");
	abort();

    case 0: /* Processus fils qui excute la commande */

	execlp("who", "who", 0);
	/* On ne doit pas passer ici */
	perror("exit who");
	exit(1);

    default: /* Le pre attend */
	wait(0);
	break;	   
    }

    /* Excution de "cd" directemnt :
     * On ne peut pas faire comme ci-dessus avec fork()/exec() car
     *
     *	1. il n'existe pas de /bin/cd ou autre excutable adquat...
     *	2. ... car, s'il existait, le changement de rpertoire se produirait
     *	   uniquement dans le processus fils, et serait donc sans effet sur
     *	   les commandes suivantes.
     *
     * 'cd' est donc une de ces commandes *internes* (built-in) que le shell
     * doit excuter directement (sans fork()). C'est le cas de toutes les
     * commandes qui modifient les attributs du processus.
     */
    home = getenv("HOME");
    if (home == 0 || strcmp(home, "") == 0)
    {
	fprintf(stderr, "Cannot locate home directory\n");
	exit(1);
    }
    if (chdir(getenv("HOME")) < 0)
    {
	perror("chdir");
	exit(1);
    }

    /* Excution de "ls -ls" :
     * on revient au schma normal
     */
    switch (fork())
    {
    case -1:
	perror("fork");
	abort();

    case 0: /* Processus fils qui excute la commande */

	execlp("ls", "ls", "-ls", 0);
	/* On ne doit pas passer ici */
	perror("exit ls");
	exit(1);

    default: /* Le pre attend */

	wait(0);
	break;	   
    }

    /* Fin du programme */
    exit(0);

}


